import logging
from typing import Dict, Any
from prefect import flow, task
from pyspark.sql import DataFrame
from pyspark.sql.functions import when, lit, sum as py_sum, col, count

from Utils.SparkUtils import createSession, get_goal_by_kpi, get_execute_by_service

logger = logging.getLogger()


class CommissionProcess:

    def __init__(self, config: Dict[str, Any]) -> None:
        self.conf = config
        self.identifier = self.conf["identifier"]
        self.period = self.conf["period"]
        self.session = None

        self.agents = []
        self.periods = []
        self.inputs = {}
        self.inter_results = {}

    def init(self, spark_jars: Dict[str, str], mongodb_uri: str = "") -> None:
        self.session = createSession(self.identifier, spark_jars, mongodb_uri)

    @task
    def get_inputs(self) -> None:
        try:
            inputs = self.conf["inputs"]
            for input_obj in inputs["data"]:
                identifier = input_obj["identifier"]
                df_input = self.session.read.format("com.mongodb.spark.sql.DefaultSource").\
                           option("collection", identifier).load()
                self.inputs.update({identifier: df_input})
        except Exception as e:
            raise AssertionError(f"Error in function 'get_inputs'. {e}")

    @task
    def get_agents_and_period(self, identifier: str) -> None:
        try:
            # Obtener los agentes
            agents_df = self.inputs[identifier].groupBy("AGENTE_COMISIONA").count()
            self.agents = [row[0] for row in agents_df.select("AGENTE_COMISIONA").collect() if row[0]]

            # Obtener los periodos
            period_df = self.inputs[identifier].groupBy("PERIODO_PROCESO_CODIGO").count()
            self.periods = [row[0] for row in period_df.select("PERIODO_PROCESO_CODIGO").collect()]

        except Exception as e:
            raise AssertionError(f"Error obteniendo agentes. {e}")

    @task
    def filter_data(self, identifier: str) -> None:
        try:
            this_df = self.inputs[identifier]
            this_df = this_df.filter(this_df["SERVICIO"] == "Postpaid")
            this_df = this_df.filter(this_df["FLAG_COMISIONABLE"] == "1")
            this_df = this_df.filter(this_df["ACTIVE_USER_TRAFFIC"] == "1")
            self.inputs[identifier] = this_df
        except Exception as e:
            raise AssertionError(f"Error filtrando datos. {e}")

    @task
    def get_goals_second_way(self, gross_name: str, goals_name: str) -> DataFrame:
        result = None
        try:
            goal_df = self.inputs[goals_name]
            gross_df = self.inputs[gross_name]

            goal_df = goal_df.withColumn("META_1", when(goal_df["KPI"] == "EQUIPOS", goal_df["META_FINAL"]).otherwise(lit(0)))
            goal_df = goal_df.withColumn("META_2", when(goal_df["KPI"] == "GROSS B2B", goal_df["META_FINAL"]).otherwise(lit(0)))
            goal_df = goal_df.withColumn("META_3", when(goal_df["KPI"] == "GROSS POSPAGO", goal_df["META_FINAL"]).otherwise(lit(0)))

            sums = [py_sum(column).alias(column) for column in ["META_1", "META_2", "META_3"]]
            goal_group = goal_df.groupBy("CEDULA", "PERIODO_PROCESO_CODIGO").agg(*sums)

            gross_df_agent = gross_df.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").count(). \
                select("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO")
            goal_group = goal_group.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == goal_group["CEDULA"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == goal_group["PERIODO_PROCESO_CODIGO_2"]]

            result = gross_df_agent.join(goal_group, condition, how='left')
            result = result.na.fill(value=0, subset=["META_1", "META_2", "META_3"])
            result = result.drop(*["CEDULA", "PERIODO_PROCESO_CODIGO_2"])
        except Exception as e:
            raise AssertionError(f"Error obteniendo metas. Segunda forma. {e}")
        finally:
            return result

    @task
    def get_executed_second_way(self, gross_name: str, team_name: str) -> DataFrame:
        result = None
        try:
            team_df = self.inputs[team_name]
            gross_df = self.inputs[gross_name]

            executed1 = team_df[team_df["MODELO_TIPO"] == "SMARTPHONE"]
            executed1 = executed1.groupBy("CONSULTOR_DOCUMENTO", "PERIODO_PROCESO_CODIGO").agg(count("MODELO_TIPO").
                                                                                               alias("EJECUTADO_1"))
            gross_df_agent = gross_df.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").count(). \
                select("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO")
            executed1 = executed1.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == executed1["CONSULTOR_DOCUMENTO"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == executed1["PERIODO_PROCESO_CODIGO_2"]]
            result_1 = gross_df_agent.join(executed1, condition, how='left')
            result_1 = result_1.na.fill(value=0, subset=["EJECUTADO_1"])
            result_1 = result_1.drop(*["CONSULTOR_DOCUMENTO", "PERIODO_PROCESO_CODIGO_2"])

            gross_b2b = gross_df[(gross_df["SEGMENTO"] == "B2B") & (gross_df["SERVICIO"] == "Postpaid") &
                                 (gross_df["FLAG_COMISIONABLE"] == "1") & (gross_df["ACTIVE_USER_TRAFFIC"] == "1")]
            gross_b2b = gross_b2b.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").agg(count("SERVICIO").
                                                                                            alias("EJECUTADO_2"))
            gross_b2b = gross_b2b.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            gross_b2b = gross_b2b.withColumnRenamed("AGENTE_COMISIONA", "AGENTE_COMISIONA_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == gross_b2b["AGENTE_COMISIONA_2"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == gross_b2b["PERIODO_PROCESO_CODIGO_2"]]
            result_2 = gross_df_agent.join(gross_b2b, condition, how='left')
            result_2 = result_2.na.fill(value=0, subset=["EJECUTADO_2"])
            result_2 = result_2.drop(*["AGENTE_COMISIONA_2", "PERIODO_PROCESO_CODIGO_2"])

            gross_b2c = gross_df[(gross_df["SEGMENTO"] == "B2C") & (gross_df["SERVICIO"] == "Postpaid") &
                                 (gross_df["FLAG_COMISIONABLE"] == "1") & (gross_df["ACTIVE_USER_TRAFFIC"] == "1")]
            gross_b2c = gross_b2c.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").agg(count("SERVICIO").
                                                                                            alias("EJECUTADO_3"))
            gross_b2c = gross_b2c.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            gross_b2c = gross_b2c.withColumnRenamed("AGENTE_COMISIONA", "AGENTE_COMISIONA_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == gross_b2c["AGENTE_COMISIONA_2"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == gross_b2c["PERIODO_PROCESO_CODIGO_2"]]
            result_3 = gross_df_agent.join(gross_b2c, condition, how='left')
            result_3 = result_3.na.fill(value=0, subset=["EJECUTADO_3"])
            result_3 = result_3.drop(*["AGENTE_COMISIONA_2", "PERIODO_PROCESO_CODIGO_2"])

            result = result_1.join(result_2, ["AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO"], 'left')
            result = result.join(result_3, ["AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO"])
        except Exception as e:
            logger.error(f"Error obteniendo ejecutados. Segunda forma. {e}")
        finally:
            return result

    @task
    def get_source_value(self, gross_name: str, base_name: str) -> DataFrame:
        result = None
        try:
            base_df = self.inputs[base_name]
            gross_df = self.inputs[gross_name]

            base_df = base_df.select("PERIODO_PROCESO_CODIGO", "CEDULA", "VARIABLE_COMISION")
            base_df = base_df.dropDuplicates()

            gross_df_agent = gross_df.groupBy("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO").count(). \
                select("AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO")
            base_df = base_df.withColumnRenamed("PERIODO_PROCESO_CODIGO", "PERIODO_PROCESO_CODIGO_2")
            condition = [gross_df_agent["AGENTE_COMISIONA"] == base_df["CEDULA"],
                         gross_df_agent["PERIODO_PROCESO_CODIGO"] == base_df["PERIODO_PROCESO_CODIGO_2"]]
            result = gross_df_agent.join(base_df, condition, how='left')
            result = result.na.fill(value=0, subset=["VARIABLE_COMISION"])
            result = result.drop(*["CEDULA", "PERIODO_PROCESO_CODIGO_2"])
        except Exception as e:
            logger.error(f"Error obteniendo valor de monto de origen. {e}")
        finally:
            return result

    @task
    def get_commission_per_agent(self, df_goals: DataFrame, df_executes: DataFrame, df_base: DataFrame) -> DataFrame:
        merged = None
        try:
            merged = df_goals.join(df_executes, ["AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO"], 'left')
            merged = merged.join(df_base, ["AGENTE_COMISIONA", "PERIODO_PROCESO_CODIGO"], 'left')
            merged = merged.withColumn("CUMPLIMIENTO EQUIPOS",
                                       when(merged["META_1"] != 0, merged["EJECUTADO_1"]/merged["META_1"]).
                                       otherwise(lit(0)))
            merged = merged.withColumn("PONDERADO EQUIPOS", merged["CUMPLIMIENTO EQUIPOS"] * 0.25)
            merged = merged.withColumn("CUMPLIMIENTO B2B",
                                       when(merged["META_2"] != 0, merged["EJECUTADO_2"]/merged["META_2"]).
                                       otherwise(lit(0)))
            merged = merged.withColumn("PONDERADO B2B", merged["CUMPLIMIENTO B2B"] * 0.1)
            merged = merged.withColumn("CUMPLIMIENTO B2C",
                                       when(merged["META_3"] != 0, merged["EJECUTADO_3"]/merged["META_3"]).
                                       otherwise(lit(0)))
            merged = merged.withColumn("PONDERADO B2C", merged["CUMPLIMIENTO B2C"] * 0.65)
            merged = merged.withColumn("PONDERADO TOTAL", merged["PONDERADO EQUIPOS"] + merged["PONDERADO B2B"] + merged["PONDERADO B2C"])
            merged = merged.withColumn("FACTOR", when(merged["PONDERADO TOTAL"] <= 0.79, lit(0)).
                                       when((merged["PONDERADO TOTAL"] >= 0.8) & (merged["PONDERADO TOTAL"] < 2), merged["PONDERADO TOTAL"]).
                                       when(merged["PONDERADO TOTAL"] >= 2, lit(2)))
            merged = merged.withColumn("COMISION", merged["FACTOR"]*merged["VARIABLE_COMISION"])

            merged = merged.withColumnRenamed("EJECUTADO_2", "EJECUTADO B2B")
            merged = merged.withColumnRenamed("EJECUTADO_3", "EJECUTADO B2C")
            merged = merged.withColumnRenamed("EJECUTADO_1", "EJECUTADO Cant Smart")
            merged = merged.withColumnRenamed("META_1", "META EQUIPOS")
            merged = merged.withColumnRenamed("META_2", "META GROSS B2B")
            merged = merged.withColumnRenamed("META_3", "META GROSS POSPAGO")
            merged = merged.withColumnRenamed("VARIABLE_COMISION", "VARIABLE COMISION (monto de origen)")
        except Exception as e:
            logger.error(f"Error operando la comisión. {e}")
        finally:
            return merged

    @task
    def write_result(self, df: DataFrame, collection_name: str) -> None:
        try:
            df.write.format("com.mongodb.spark.sql.DefaultSource").option("collection", collection_name). \
                mode("append").save()
        except Exception as e:
            logger.error(f"Error guardando datos en BD. {e}")
